package com.jiuqi.mde;

import com.jiuqi.util.DuplicationException;
import com.jiuqi.util.RecIndex;
import java.util.ArrayList;
import java.util.List;

/* loaded from: com_jiuqi_mde.jar:com/jiuqi/mde/MDERecordSet.class */
public class MDERecordSet {
    protected MDEDataInfo dataInfo;
    protected MDEFieldSet fields;
    protected boolean hasOrder;
    protected ArrayList records = new ArrayList(40);
    protected MDEIndex dimValuesIndex = null;
    protected ArrayList additionalIndexes = new ArrayList();
    private boolean isUseLoadDataSync = false;
    protected Object mutexLoadData = new Object();
    protected Object mutexBuildIndex = new Object();

    public MDERecordSet(MDEDataInfo dataInfo) {
        this.dataInfo = null;
        this.fields = null;
        this.hasOrder = false;
        this.dataInfo = dataInfo;
        this.fields = dataInfo.fields();
        this.hasOrder = dataInfo.dimensions().hasOrder();
    }

    public void setIsUseLoadDataSync(boolean flag) {
        this.isUseLoadDataSync = flag;
    }

    public boolean getIsUseLoadDataSync() {
        return this.isUseLoadDataSync;
    }

    public final String getName() {
        return this.dataInfo.getName();
    }

    public final MDEDataInfo getDataInfo() {
        return this.dataInfo;
    }

    public final int size() {
        return this.records.size();
    }

    public final MDERecord get(int index) {
        return (MDERecord) this.records.get(index);
    }

    public final void add(MDERecord record) throws DuplicationException, MDEException {
        if (this.dimValuesIndex != null && (record.dimValues != null || !record.isInserted())) {
            if (this.hasOrder) {
                this.dimValuesIndex.genOrder(record.getDimValues());
            }
            addToDVIndex(record);
        }
        this.records.add(record);
    }

    public final void remove(int index) {
        MDERecord record = (MDERecord) this.records.get(index);
        if (this.dimValuesIndex != null && record.dimValues != null) {
            this.dimValuesIndex.remove(record.getDimValues());
        }
        record.emptyRecord();
        this.records.remove(index);
    }

    public final void remove(MDERecord record) {
        if (this.dimValuesIndex != null && record.dimValues != null) {
            this.dimValuesIndex.remove(record.getDimValues());
        }
        record.emptyRecord();
        this.records.remove(record);
        if (this.additionalIndexes != null) {
            this.additionalIndexes.clear();
        }
    }

    public final void removeAll(List removeRecords) {
        for (int i = 0; i < removeRecords.size(); i++) {
            remove((MDERecord) removeRecords.get(i));
        }
    }

    protected final void addToDVIndex(MDERecord record) throws DuplicationException, MDEException {
        record.refreshDimValues(null);
        if (!this.dataInfo.isUnique()) {
            this.dimValuesIndex.addlast(record.getDimValues(), record);
            return;
        }
        try {
            this.dimValuesIndex.genOrder(record.getDimValues());
            this.dimValuesIndex.add(record.getDimValues(), record);
        } catch (DuplicationException ex) {
            if (this.hasOrder) {
                record.getDimValues().setValue(MDEDimension.ORDER_NAME, (Object) null);
                this.dimValuesIndex.genOrder(record.getDimValues());
                this.dimValuesIndex.add(record.getDimValues(), record);
                return;
            }
            throw ex;
        }
    }

    /* JADX INFO: Access modifiers changed from: protected */
    /* JADX WARN: Multi-variable type inference failed */
    /* JADX WARN: Type inference failed for: r0v4, types: [java.lang.Object] */
    /* JADX WARN: Type inference failed for: r0v5, types: [java.lang.Throwable] */
    /* JADX WARN: Type inference failed for: r0v8 */
    public final void buildDimValuesIndex() throws MDEException {
        if (getIsUseLoadDataSync()) {
            ?? r0 = this.mutexBuildIndex;
            synchronized (r0) {
                if (this.dimValuesIndex == null) {
                    buildDimValuesIndex0();
                }
                r0 = r0;
                return;
            }
        }
        buildDimValuesIndex0();
    }

    private void buildDimValuesIndex0() throws MDEException {
        if (this.dataInfo.dimensions().size() == 0) {
            throw new MDEException("没有定义维度，无法使用键值查找功能");
        }
        if (this.dimValuesIndex == null) {
            this.dimValuesIndex = new MDEIndex();
        } else {
            this.dimValuesIndex.clear();
        }
        try {
            int count = this.records.size();
            for (int i = 0; i < count; i++) {
                MDERecord record = (MDERecord) this.records.get(i);
                addToDVIndex(record);
            }
        } catch (DuplicationException ex) {
            throw new MDEException((Exception) ex);
        }
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public final MDERecord dvFind(MDEDimValueSet dimValues) throws MDEException {
        if (this.dimValuesIndex == null) {
            buildDimValuesIndex();
        }
        return (MDERecord) this.dimValuesIndex.findkey(dimValues);
    }

    protected final MDERecord dvFindNoOrder(MDEDimValueSet dimValues) throws MDEException {
        if (this.dimValuesIndex == null) {
            buildDimValuesIndex();
        }
        if (this.hasOrder) {
            int index = this.dimValuesIndex.findNoOrder(dimValues);
            return (MDERecord) this.dimValuesIndex.getValue(index);
        }
        return (MDERecord) this.dimValuesIndex.findkey(dimValues);
    }

    /* JADX INFO: Access modifiers changed from: protected */
    public final List dvFindMatches(MDEDimValueSet speciafication) throws MDEException {
        List result;
        if (this.dimValuesIndex == null) {
            buildDimValuesIndex();
        }
        if (getIsUseLoadDataSync()) {
            RecIndex recIndex = this.dimValuesIndex;
            synchronized (recIndex) {
                result = dvFindMatches0(speciafication);
                recIndex = recIndex;
            }
        } else {
            result = dvFindMatches0(speciafication);
        }
        return result;
    }

    private List dvFindMatches0(MDEDimValueSet speciafication) {
        int start = 0;
        int stop = this.dimValuesIndex.size();
        boolean allSpeciafied = false;
        if (speciafication.dimensions().equals(this.dataInfo.dimensions()) && speciafication.getValue(0) != null) {
            int index = -1;
            int i = 0;
            int count = speciafication.size();
            while (true) {
                if (i >= count) {
                    break;
                }
                if (speciafication.getValue(i) != null) {
                    i++;
                } else {
                    index = i;
                    break;
                }
            }
            allSpeciafied = index < 0;
            if (allSpeciafied) {
                start = this.dimValuesIndex.searchfirst(speciafication);
                if (start < 0) {
                    start = stop;
                } else {
                    stop = this.dimValuesIndex.searchlast(speciafication);
                    if (stop < this.dimValuesIndex.size()) {
                        stop++;
                    }
                }
            } else {
                MDEDimension dimension = speciafication.dimensions().get(index);
                MDEDimValueSet find = (MDEDimValueSet) speciafication.clone();
                find.setValue(index, dimension.getMinValue());
                start = this.dimValuesIndex.searchfirst(find);
                if (start < 0) {
                    start = (-1) - start;
                }
                find.setValue(index, dimension.getMaxValue());
                int stop2 = this.dimValuesIndex.searchlast(find);
                if (stop2 < 0) {
                    stop = (-1) - stop2;
                } else {
                    stop = stop2 + 1;
                }
                if (stop > this.dimValuesIndex.size()) {
                    stop--;
                }
            }
        }
        List result = new ArrayList(stop - start);
        for (int i2 = start; i2 < stop; i2++) {
            MDERecord record = (MDERecord) this.dimValuesIndex.getValue(i2);
            if (allSpeciafied || record.getDimValues().matches(speciafication)) {
                result.add(record);
            }
        }
        return result;
    }

    protected final void rebuildFieldIndex(MDEAdditionalIndex index) {
        index.clear();
        index.setCapacity(this.records.size());
        MDEField field = index.getKeyFields().get(0);
        int count = this.records.size();
        for (int i = 0; i < count; i++) {
            MDERecord record = (MDERecord) this.records.get(i);
            index.addlast(record.getFieldValue(field), record);
        }
    }

    protected MDEAdditionalIndex createAdditionalIndex(MDEField field) {
        MDEAdditionalIndex result = new MDEAdditionalIndex();
        result.setField(field);
        return result;
    }

    protected final MDEAdditionalIndex getAdditionalIndex(MDEField field) {
        MDEAdditionalIndex result = null;
        int i = 0;
        int count = this.additionalIndexes.size();
        while (true) {
            if (i >= count) {
                break;
            }
            MDEAdditionalIndex index = (MDEAdditionalIndex) this.additionalIndexes.get(i);
            if (!index.keyFieldsMatch(field)) {
                i++;
            } else {
                result = index;
                break;
            }
        }
        if (result == null) {
            result = createAdditionalIndex(field);
            rebuildFieldIndex(result);
            this.additionalIndexes.add(result);
        }
        return result;
    }

    protected final MDERecord fvFind(MDEField field, Object fieldValue) {
        MDEAdditionalIndex index = getAdditionalIndex(field);
        MDERecord rec = (MDERecord) index.findkey(fieldValue);
        return rec;
    }

    public void resetRecords() {
        if (this.additionalIndexes != null) {
            this.additionalIndexes.clear();
        }
        if (this.dimValuesIndex != null) {
            this.dimValuesIndex.clear();
            this.dimValuesIndex = null;
        }
        this.records.clear();
    }
}
